package pacman3d.labyrinth.cells;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;

import org.w3c.dom.*;

import pacman3d.util.*;

/**
 * <b>Titel:</b>Pacman 3D - Labyrinthzellen<p>
 *
 * <b>Beschreibung:</b><br>
 * <p>
 *
 * <b>Copyright:</b>	Copyright (c) 2001/02<br>
 *
 * @author              Labyrinth-Gruppe<br>
 * @version             1.0 03/15/2002<br>
 */
public class CellSimpleCone extends Cell { // Der Java3D Knoten
  private TransformGroup view;
  // Die Position der Zelle
  private Tuple3i position;
  // Die Seiten die nicht gezeichnet werden

  protected Cone cone;
  /** Leerer Konstruktor
   *  initialisiert die Zelle
   */
  public CellSimpleCone() {
    super(false);
    cone = this.getNode();

  }

  /** Konstruktor
   *  @param pos Position der Zelle
   */
  public CellSimpleCone(Tuple3i pos) {
    super(false);
    cone = this.getNode();
    setPosition(pos);
  }

  /** Methode setzt die Position der Zelle
   *  @param pos Position der Zelle
   */
  public void setPosition(Tuple3i pos) {
    position = pos;
  }

  /**
   * Methode liefert die gesetzte Position der Zelle im Labyrinth.
   *
   * @return Position der Zelle im Labyrinth.
   */
  public Tuple3i getPosition() {
    return position;
  }

  /** Methode setzt die Seiten Maske der Zelle
   *  @param sideMask  Flags der Seiten die <b>nicht</b> erzeugt werden sollen
   *                   (Teil)Summe der folgenden Flags:
   *                   SIDE_RIGHT, SIDE_FRONT, SIDE_LEFT, SIDE_BACK
   */
  public void setSideMask(int sideMask) {
  }

  /** Gibt den Java3D Knoten zurck
   *  @return Den Java3d Knoten mit der Ansicht der Zelle.
   */
  public javax.media.j3d.Node getJ3DGroup() {
    build();
    return view;
  }


  /** Erzeugt den Java3D Knoten
   */
  private void build() {
    Transform3D tr = new Transform3D();;

    if(position != null)
      tr.setTranslation(new Vector3f(position.x,position.y,position.z));

    view = new TransformGroup(tr);
    view.addChild(cone.cloneTree(true));
  }

 /** Setzt eine Textur auf eine Seite
   *  @param side               Flag der Seite(n), die Texturiert werden soll(en).
   *                            Fr mehrere Seiten Summe der Flags.
   *  @param sTextureType       Der Name des Texturtyps, wie er fuer den
   *                            Zugriff auf den <code>ResourceHandler</code>
   *                            benoetigt wird.
   *  @param texId              Die Textur-Id
   *  @see                      pacman3d.util.ResourceHandler
   */
  public void setTexture(int side, String sTextureName, long texId) throws Exception {
    // Textur vom ResourceHandler holen
    ResourceHandler oResourceHandler = ResourceHandler.getInstance();
    Texture tex = oResourceHandler.getTexture( sTextureName );

    String sNull = (tex == null) ? "texture " + sTextureName +
        " not available" : "texture " + sTextureName + " loaded";
    Debug.out (getClass().getName(), sNull, Debug.LEVEL_NOTICE);

    Appearance app = new Appearance();
    app.setCapability(Appearance.ALLOW_TEXTURE_READ);
    // Textur attribute
    TextureAttributes texAttr = new TextureAttributes();
    texAttr.setTextureMode(TextureAttributes.MODULATE);
    app.setTextureAttributes(texAttr);
    app.setTexture(tex);
    // Farbe und Material werden gesetzt
    app.setMaterial(new Material(white,black,white,white,10.0f));
    cone.setAppearance(app);
  }

  /** Setzt eine Farbe auf eine Seiten
   *  @param side   Flag der Seite(n), die Coloriert werden soll(en).
   *                Fr mehrere Seiten Summe der Flags.
   *  @param color  Die Farbe
   *  @param texId  Die Textur-Id
   */
  public void setColor(int side, Color3f color) {
    Material material = new Material(color,black,white,white,10.0f);
    material.setCapability(Material.ALLOW_COMPONENT_READ);
    material.setLightingEnable(true);
    Appearance appearance = new Appearance();
    appearance.setCapability(Appearance.ALLOW_MATERIAL_READ);
    appearance.setMaterial(material);

    cone.setAppearance(appearance);
  }

  /** Erzeugt eine Seite
   */
  private Cone getNode() {
    Cone cone = new Cone(0.3f, 1.0f, Cone.GENERATE_TEXTURE_COORDS |
                         Cone.GENERATE_NORMALS | Cone.ENABLE_APPEARANCE_MODIFY, null);
    cone.setNumTriangles(5);
    return cone;
  }

  /**
   * liefert ein Array der Appearance-Objekte der sechs Seiten einer CellWall zurck
   * @return ein sechselementiges Array der Appearance-Objekte
   */
  public Appearance[] getAppearance() {
    Appearance[] appearance = new Appearance[1];
    appearance[0] = cone.getAppearance();

    return appearance;
  }

    /**
   * Nimmt ein XML Element entgegen, und deserialisiert den Zustand des
   * eigenen Objektes.
   *
   * @param oElement      Das XML-Element, unterhalb dessen ein Objekt
   *                      seinen Zustand speichern darf.
   */
  public void loadFromDOM( org.w3c.dom.Element oElement ) {
    // zunchst die Elternmethode aufrufen und sie werkeln lassen
    super.loadFromDOM(oElement);

    // wenn kein Elemenmt bergeben wurde, dann sofort zurckspringen
    if (oElement == null) {
      return;
    }

    // wir brauchen die Instanz des ResourcenHandlers zum Holen von Texturen
    ResourceHandler oResourceHandler = pacman3d.util.ResourceHandler.getInstance();

    // jetzt holen wir uns unsere Parameter fr die Texturen (entweder ist
    // das bergebene oElement selbst <params> oder ein Kind von ihm)
    Element elParams = null;
    if (oElement.getTagName().equals(LevelXmlSyntax.TAG_PARAMS)) {
      elParams = oElement;
    }
    else {
      elParams = XmlUtils.getChildElement(oElement, LevelXmlSyntax.TAG_PARAMS);
    }
    if (elParams != null) {
      if (elParams.hasChildNodes()) {
        NodeList oChildren = elParams.getChildNodes();

        // gehe durch alle Parameter durch und lade Texturen ein
        for (int i=0; i < oChildren.getLength(); i++) {
          org.w3c.dom.Node ndCurrent = oChildren.item(i);
          if (ndCurrent.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
            if (((Element)ndCurrent).getTagName().equals("texture")) {

              // <texture side="1"><rsrclink xlink:type="simple" xlink:href="#res_wallX"/></texture>

              // job 1 ist das Holen der Seitennummer (iSide)
              int iSide = 0;
              try {
                String sSide = ((Element)ndCurrent).getAttribute("side");
                if (sSide != null) {
                  iSide = new Integer(sSide).intValue();
                }
              }
              catch (java.lang.NumberFormatException ex) {
                Debug.out (this.getClass().getName(), "keine Zahl vorgefunden", Debug.LEVEL_WARNING);
              }

              // job 2 ist das Holen des Texturnamens
              Element elResLink = XmlUtils.getChildElement((Element)ndCurrent, LevelXmlSyntax.TAG_RESOURCELINK);
              String sResName = "";
              if (elResLink != null) {
                // hole den Namen der Resourcen aus dem <rsrclink>-Element und schneide
                // das fhrende # des Namens ab (im <rscrlink>-Element ist der Name der
                // Resource im href-Attribut als ein XPointer-Verweis auf eine folgende
                // Resourcendefinition abgelegt. Die Syntax fr einen solchen lokalen
                // Verweis ist: '#' Name. Dementspreched mu das fhrende # abgeschnitten
                // werden, bevor der ResourceHandler die Anweisung zum Laden bekommt
                sResName = elResLink.getAttribute(LevelXmlSyntax.ATTR_RESOURCELINK_HREF);
                if (sResName.startsWith("#")) {
                  sResName = sResName.substring(1);
                }
                // Debug.out(this.getClass().getName(), "" + sResName);

                // lade die Textur vom ResourcenHandler
                // javax.media.j3d.Texture2D txLoaded = null;
                // txLoaded = oResourceHandler.getTexture(sResName);
                // String sNull = (txLoaded == null) ? "texture " + sResName + " not available" : "texture " + sResName + " loaded";
                // Debug.out (this.getClass().getName(), sNull, Debug.LEVEL_NOTICE);

                try {
                  this.setTexture(iSide, sResName, 0);
                }
                catch (java.lang.Exception ex) {
                  Debug.out (getClass().getName(), ex, Debug.LEVEL_WARNING);
                  Debug.out (getClass().getName(), "error assigning texture", Debug.LEVEL_WARNING);
                }
              }
            }
          }
        }
      }
    }
  }



  /**
   * Serialisiert den Zustand der Instanz in ein XML-Element, hier insbesondere
   * die Parameter zur Darstellung der Zelle (Texturen)
   *
   * @param oDoc das Document, in das die Daten eingefgt werden sollen
   * (wird fr das Erzeugen von Elementen nach dem Factory-Konzept bentigt)
   * @return Valide XML-Daten in String-Form.
   */
  public org.w3c.dom.Element saveToDOM(org.w3c.dom.Document oDoc) {
    // zunchst die Elternmethode aufrufen und sie werkeln lassen
    Element oCell = super.saveToDOM(oDoc);

    // Einfgen der Parameter (hier mu eine Prfung rein, ob schon
    // ein <params>-Tag vorhanden ist
    Element elParams = oDoc.createElement(LevelXmlSyntax.TAG_PARAMS);

    // wir brauchen die Instanz des ResourcenHandlers zum Ablegen von Texturen
    ResourceHandler oResourceHandler = pacman3d.util.ResourceHandler.getInstance();

    // die Adresse der Textur der aktuellen Seite holen{
    // String sTextureName = sides[iSide].getAppearance().getTexture().toString();
    Debug.out(this.getClass().getName(), "cone == " + cone);
    Debug.out(this.getClass().getName(), "cone.getAppearance() == " + cone.getAppearance());
    Debug.out(this.getClass().getName(), "cone.getAppearance().getTexture() == " + cone.getAppearance().getTexture());
    String sTextureName = oResourceHandler.addTexture( cone.getAppearance().getTexture() );
    // oResourceHandler.addTexture(sides[iSide].getAppearance().getTexture());

    // eine <texture side="."><rsrclink ... xlink:href=""/></texture> Struktur aufbauen
    Element elTexture = oDoc.createElement (LevelXmlSyntax.TAG_TEX);
    elTexture.setAttribute(LevelXmlSyntax.ATTR_TEX_SIDE, "" + (1 << 2/*iSide*/));
    Element elResourceLink = oDoc.createElement(LevelXmlSyntax.TAG_RESOURCELINK);
    elResourceLink.setAttribute("xlink:type", "simple");
    elResourceLink.setAttribute(LevelXmlSyntax.ATTR_RESOURCELINK_HREF, "#" + sTextureName);
    elTexture.appendChild(elResourceLink);
    elParams.appendChild(elTexture);



    oCell.appendChild(elParams);

    return oCell;
  }

  /**
   * klont das Objekt und liefert es zurck
   * @return der Klon der Instanz mit dem gleichen Typ (aber per
   * Definition als Objekt gecastet)
   */
  public Object clone () {
    return super.clone();
  }


}